import { ChangeDetectorRef, Component, Directive, ElementRef, HostListener, Injectable, Input, NgModule, TemplateRef, ViewChild, ViewContainerRef } from '@angular/core';
import { BehaviorSubject as BehaviorSubject$1 } from 'rxjs/BehaviorSubject';
import { Observable } from 'rxjs';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
let nextId = 0;
class PopoverEntry {
    /**
     * @param {?} tempRef
     */
    constructor(tempRef) {
        this.anchorSide = new BehaviorSubject$1('bottom');
        this.xPos = new BehaviorSubject$1(0);
        this.yPos = new BehaviorSubject$1(0);
        this.isInteractive = false;
        this.delay = 0;
        this.isShown = false;
        this.isActive = false;
        this.template = tempRef;
        nextId++;
        this._id = nextId;
    }
    /**
     * @return {?}
     */
    get id() { return this._id; }
    /**
     * @return {?}
     */
    destroy() {
        this.anchorSide.complete();
        this.xPos.complete();
        this.yPos.complete();
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class GistPopoverService {
    constructor() {
        this._popovers = [];
        this.idIndexMap = {};
    }
    /**
     * @param {?} templateRef
     * @return {?}
     */
    registerPopover(templateRef) {
        let /** @type {?} */ entry = new PopoverEntry(templateRef);
        this.idIndexMap[entry.id] = this._popovers.length;
        this._popovers.push(entry);
        return entry;
    }
    /**
     * @param {?} entry
     * @return {?}
     */
    unRegisterPopover(entry) {
        let /** @type {?} */ targetIndex = this.idIndexMap[entry.id];
        this._popovers.splice(targetIndex, 1);
        entry.destroy();
    }
}
GistPopoverService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
GistPopoverService.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
let offset = 15;
let eleMoveTrackTime = 1000 / 30;
const ɵ0 = (val) => {
    let /** @type {?} */ opts = ['ele', 'mouse']; // or x,y where x and y are numbers
    let /** @type {?} */ isValid = false;
    if (opts.indexOf(val) !== -1) {
        isValid = true;
    }
    else if (val.indexOf(',')) {
        let /** @type {?} */ coords = val.split(',');
        if (coords.length === 2) {
            let /** @type {?} */ isXnan = isNaN(parseFloat(coords[0]));
            let /** @type {?} */ isYnan = isNaN(parseFloat(coords[1]));
            isValid = !isXnan && !isYnan;
        }
    }
    if (!isValid) {
        console.error('gistPopover invalid paramater value. Target must be one of ' + opts.join(', ') + ', XX,YY ( as page coordinates). Value given: ' + val);
        return;
    }
    return val;
};
const ɵ1 = (val) => {
    let /** @type {?} */ opts = ['top', 'bottom', 'left', 'right'];
    if (opts.indexOf(val) === -1) {
        console.error('gistPopover invalid paramater value. Position must be one of ' + opts.join(', ') + '. Value given: ' + val);
        return;
    }
    return val;
};
const ɵ2 = (val) => {
    if (val === 'true') {
        return true;
    }
    else if (val === 'false') {
        return false;
    }
    else {
        console.error('gistPopover invalid paramater value. Position must be the string "true" or "false". Value given: ' + val);
    }
};
const ɵ3 = (val) => {
    if (val === 'true') {
        return true;
    }
    else if (val === 'false') {
        return false;
    }
    else if (val === 'undefined') {
        return undefined;
    }
    else {
        console.error('gistPopover invalid paramater value. Position must be the string "true", "false", or "undefined". Value given: ' + val);
        return;
    }
};
const ɵ4 = (val) => {
    let /** @type {?} */ tmp = +val;
    if (tmp.toString() !== val.toString() || tmp < 0) {
        console.error('gistPopover invalid paramater value. Delay must be a number >= 0');
        return;
    }
};
let validInputs = [
    {
        key: 'target',
        default: 'ele',
        parse: ɵ0,
    },
    {
        key: 'position',
        default: 'bottom',
        parse: ɵ1
    },
    {
        key: 'isInteractive',
        default: false,
        parse: ɵ2
    },
    {
        key: 'isVisible',
        default: undefined,
        parse: ɵ3
    },
    {
        key: 'delay',
        default: 100,
        parse: ɵ4
    }
];
class GistPopoverDirective {
    /**
     * @param {?} templateRef
     * @param {?} viewContainer
     * @param {?} popService
     * @param {?} eleRef
     * @param {?} cdr
     */
    constructor(templateRef, viewContainer, popService, eleRef, cdr) {
        this.templateRef = templateRef;
        this.viewContainer = viewContainer;
        this.popService = popService;
        this.eleRef = eleRef;
        this.cdr = cdr;
        this.windowSubs = [];
        this.mouseSubs = [];
        this.isReady = false;
        this.hideTimer = undefined;
        this.showTimer = undefined;
        this.checkTimer = undefined;
        const /** @type {?} */ me = this;
        me.entry = me.popService.registerPopover(me.templateRef);
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        let /** @type {?} */ me = this;
        me.ele = me.eleRef.nativeElement;
        // take the content out of the view
        me.viewContainer.clear();
        me.windowSubs.push(Observable.fromEvent(window.document, 'resize').subscribe((e) => me.updatePopover.apply(me)));
        me.isReady = true;
    }
    /**
     * @param {?} inputString
     * @return {?}
     */
    set gistPopover(inputString) {
        let /** @type {?} */ me = this;
        let /** @type {?} */ inputs = inputString ? inputString.split(';') : [];
        validInputs.forEach(vi => { me[vi.key] = undefined; });
        inputs.forEach((input) => {
            // reset inputs
            let /** @type {?} */ pv = input.split(':');
            if (pv.length !== 2) {
                return;
            }
            let /** @type {?} */ prop = pv[0].trim();
            let /** @type {?} */ val = pv[1].trim();
            let /** @type {?} */ myProp = validInputs.filter((vi) => { return vi.key === prop; })[0];
            if (!myProp) {
                throw 'gistPopover invalid property: ' + prop + '. ' +
                    'Valid properties are ' + validInputs.map((vi) => { return vi.key; }).join(', ');
            }
            try {
                me[prop] = myProp.parse(val);
            }
            catch (/** @type {?} */ e) {
                console.error(e);
                me[prop] = undefined;
            }
        });
        validInputs.forEach((vi) => {
            if (me[vi.key] === undefined) {
                me[vi.key] = vi.default;
            }
        });
        me.waitTillReady(me.processInput);
    }
    /**
     * @return {?}
     */
    processInput() {
        let /** @type {?} */ me = this;
        // track mouse if isVisisble is not defiend
        if (me.target === 'mouse' || me.isVisible === undefined) {
            // start listening if were not
            if (me.mouseSubs.length === 0) {
                me.trackMouse();
            }
        }
        else {
            if (me.mouseSubs.length !== 0) {
                me.unTrackMouse();
            }
        }
        if (me.isVisible !== undefined) {
            if (me.isVisible) {
                me.showPopover();
            }
            else {
                me.hidePopover();
            }
        }
        if (me.isVisible && me.target.indexOf(',') !== -1) {
            me.updatePopover();
        }
    }
    /**
     * @return {?}
     */
    trackMouse() {
        let /** @type {?} */ me = this;
        let /** @type {?} */ pe = me.ele.parentElement;
        if (pe) {
            me.mouseSubs.push(Observable.fromEvent(pe, 'mousemove').subscribe((e) => {
                if (me.isVisible || me.isVisible === undefined) {
                    me.showPopover.apply(me, [e]);
                }
            }));
            me.mouseSubs.push(Observable.fromEvent(pe, 'mouseleave').subscribe((e) => {
                if (!me.isVisible) {
                    me.hidePopover.apply(me);
                }
            }));
        }
        else {
            console.warn('no element to listen to.');
        }
    }
    /**
     * @return {?}
     */
    unTrackMouse() {
        this.mouseSubs.forEach(s => s.unsubscribe());
    }
    /**
     * @param {?=} e
     * @return {?}
     */
    updatePopover(e) {
        if (this.entry && this.entry.isShown) {
            this.showPopover(e);
        }
    }
    /**
     * @param {?=} e
     * @return {?}
     */
    showPopover(e) {
        let /** @type {?} */ me = this;
        if (me.hideTimer) {
            window.clearTimeout(me.hideTimer);
            me.hideTimer = undefined;
        }
        /**
         * @param {?} coords
         * @return {?}
         */
        function display(coords) {
            me.entry.xPos.next(coords[0]);
            me.entry.yPos.next(coords[1]);
        }
        me.entry.delay = me.delay;
        me.entry.isInteractive = me.isInteractive;
        me.entry.anchorSide.next(me.position);
        let /** @type {?} */ coords = undefined;
        if (me.target === 'mouse') {
            if (e) {
                coords = me.getMouseAnchor(e);
            }
        }
        else if (me.target === 'ele') {
            coords = me.getEleAnchor(/** @type {?} */ (me.ele.parentElement));
        }
        else {
            coords = me.getTargetAnchor();
        }
        if (coords) {
            display(coords);
            if (!this.showTimer) {
                this.showTimer = window.setTimeout(() => {
                    me.entry.isShown = true;
                    me.showTimer = undefined;
                    this.checkTimer = window.setTimeout(() => {
                        me.showPopover.apply(me, [e]);
                    }, eleMoveTrackTime);
                    this.cdr.detectChanges();
                }, me.entry.delay);
            }
        }
        // debugger;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    getMouseAnchor(event) {
        let /** @type {?} */ xPos = event.pageX;
        let /** @type {?} */ yPos = event.pageY;
        let /** @type {?} */ xOffset = offset;
        let /** @type {?} */ yOffset = offset;
        if (this.position === 'left') {
            xOffset = offset * -1;
        }
        if (this.position === 'top') {
            yOffset = offset * -1;
        }
        return [xPos + xOffset, yPos + yOffset];
    }
    /**
     * @param {?} ele
     * @return {?}
     */
    getEleAnchor(ele) {
        let /** @type {?} */ bounds = ele.getBoundingClientRect();
        let /** @type {?} */ xPos = bounds.left + bounds.width / 2;
        let /** @type {?} */ yPos = bounds.top + bounds.height / 2;
        if (this.position === 'left') {
            xPos = bounds.left - offset;
        }
        else if (this.position === 'right') {
            xPos = bounds.right + offset;
        }
        if (this.position === 'top') {
            yPos = bounds.top - offset;
        }
        else if (this.position === 'bottom') {
            yPos = bounds.bottom + offset;
        }
        return [xPos, yPos];
    }
    /**
     * @return {?}
     */
    getTargetAnchor() {
        return this.target.split(',').map(x => +x);
    }
    /**
     * @return {?}
     */
    hidePopover() {
        let /** @type {?} */ me = this;
        if (this.showTimer !== undefined) {
            clearTimeout(this.showTimer);
            this.showTimer = undefined;
        }
        if (this.checkTimer !== undefined) {
            clearTimeout(this.checkTimer);
            this.checkTimer = undefined;
        }
        if (me.entry.isShown && !this.hideTimer) {
            this.hideTimer = window.setTimeout(() => {
                this.entry.isShown = false;
                this.hideTimer = undefined;
            }, this.entry.delay);
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.popService.unRegisterPopover(this.entry);
        this.mouseSubs.forEach(s => s.unsubscribe());
        this.windowSubs.forEach(s => s.unsubscribe());
    }
    /**
     * @param {?} cb
     * @return {?}
     */
    waitTillReady(cb) {
        let /** @type {?} */ me = this;
        if (me.isReady) {
            cb.apply(me);
        }
        else {
            window.setTimeout(() => {
                me.waitTillReady.apply(me, [cb]);
            }, 1);
        }
    }
}
GistPopoverDirective.decorators = [
    { type: Directive, args: [{ selector: '[gistPopover]' },] },
];
/** @nocollapse */
GistPopoverDirective.ctorParameters = () => [
    { type: TemplateRef, },
    { type: ViewContainerRef, },
    { type: GistPopoverService, },
    { type: ElementRef, },
    { type: ChangeDetectorRef, },
];
GistPopoverDirective.propDecorators = {
    "gistPopover": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class GistPopoverTargetComponent {
    /**
     * @param {?} popService
     */
    constructor(popService) {
        this.popService = popService;
    }
}
GistPopoverTargetComponent.decorators = [
    { type: Component, args: [{
                selector: 'gist-popover-target',
                template: `<ng-container *ngFor="let entry of popService._popovers">
  <ng-container *ngIf="entry.isShown || entry.isActive">
    <gist-popover-displayer [entry]="entry" [ngClass]="{'interactive': entry.isInteractive}"> </gist-popover-displayer>
  </ng-container>
</ng-container>
`,
                styles: [`:host {
  display: block;
  position: fixed;
  top: 0;
  bottom: 0;
  right: 0;
  left: 0;
  pointer-events: none;
}

.interactive {
  pointer-events: all;
}
`]
            },] },
];
/** @nocollapse */
GistPopoverTargetComponent.ctorParameters = () => [
    { type: GistPopoverService, },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
let windowOffset = 5;
class GistPopoverDisplayerComponent {
    /**
     * @param {?} eleRef
     * @param {?} cdr
     */
    constructor(eleRef, cdr) {
        this.eleRef = eleRef;
        this.cdr = cdr;
        this.viewTimer = undefined;
        this.anchorCoords = [0, 0];
        this.offsetCoords = [0, 0];
        this.windowOffsetCoords = [0, 0];
        this.lastCoords = [0, 0];
        this.throttle = undefined;
        this.isDrawing = false;
        this.ele = this.eleRef.nativeElement;
        this.ele.style.visibility = 'hidden';
    }
    /**
     * @return {?}
     */
    onMouseEnter() {
        this.entry.isActive = true;
        if (this.viewTimer) {
            window.clearTimeout(this.viewTimer);
            this.viewTimer = undefined;
        }
    }
    /**
     * @return {?}
     */
    onMouseLeave() {
        if (this.viewTimer === undefined) {
            this.viewTimer = window.setTimeout(() => {
                this.entry.isActive = false;
                this.viewTimer = undefined;
            }, this.entry.delay);
        }
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        let /** @type {?} */ me = this;
        me.target.createEmbeddedView(me.entry.template);
        me.entry.anchorSide.subscribe(anc => {
            if (me.anchor !== anc) {
                me.anchor = anc;
                me.throttledUpdate();
            }
        });
        me.entry.xPos.subscribe(x => {
            if (me.anchorCoords[0] !== x) {
                me.anchorCoords[0] = x;
                me.throttledUpdate();
            }
        });
        me.entry.yPos.subscribe(y => {
            if (me.anchorCoords[1] !== y) {
                me.anchorCoords[1] = y;
                me.throttledUpdate();
            }
        });
        this.cdr.detectChanges();
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        window.clearTimeout(this.checkInterval);
    }
    /**
     * @return {?}
     */
    updateOffset() {
        let /** @type {?} */ me = this;
        let /** @type {?} */ parent = me.ele.parentElement;
        if (!parent) {
            return;
        }
        if (me.anchor === 'left') {
            me.offsetCoords[0] = me.ele.clientWidth * -1;
            me.offsetCoords[1] = me.ele.clientHeight * -0.5;
        }
        else if (me.anchor === 'right') {
            me.offsetCoords[0] = 0;
            me.offsetCoords[1] = me.ele.clientHeight * -0.5;
        }
        else if (me.anchor === 'top') {
            me.offsetCoords[0] = me.ele.clientWidth * -0.5;
            me.offsetCoords[1] = me.ele.clientHeight * -1;
        }
        else if (me.anchor === 'bottom') {
            me.offsetCoords[0] = me.ele.clientWidth * -0.5;
            me.offsetCoords[1] = 0;
        }
        me.offsetCoords[0] = +me.offsetCoords[0].toFixed(2);
        me.offsetCoords[1] = +me.offsetCoords[1].toFixed(2);
    }
    /**
     * @return {?}
     */
    updateWindowOffset() {
        let /** @type {?} */ me = this;
        let /** @type {?} */ parent = me.ele.parentElement;
        if (!parent) {
            return;
        }
        let /** @type {?} */ myBounds = me.ele.getBoundingClientRect();
        let /** @type {?} */ parentBounds = parent.getBoundingClientRect();
        let /** @type {?} */ height = myBounds.height;
        let /** @type {?} */ width = myBounds.width;
        let /** @type {?} */ x1 = me.anchorCoords[0] + me.offsetCoords[0];
        let /** @type {?} */ x2 = x1 + width;
        let /** @type {?} */ y1 = me.anchorCoords[1] + me.offsetCoords[1];
        let /** @type {?} */ y2 = y1 + height;
        if (x1 < windowOffset) {
            me.windowOffsetCoords[0] = windowOffset - x1;
        }
        if (x2 > parentBounds.right - windowOffset) {
            me.windowOffsetCoords[0] = parentBounds.right - windowOffset - x2;
        }
        if (y1 < windowOffset) {
            me.windowOffsetCoords[1] = windowOffset - y1;
        }
        if (y2 > parentBounds.bottom - windowOffset) {
            me.windowOffsetCoords[1] = parentBounds.bottom - windowOffset - y2;
        }
        me.windowOffsetCoords[0] = +me.windowOffsetCoords[0].toFixed(2);
        me.windowOffsetCoords[1] = +me.windowOffsetCoords[1].toFixed(2);
        if (me.anchorCoords[1] + me.offsetCoords[1] + me.windowOffsetCoords[1] < windowOffset) {
            console.error(me.anchorCoords[1] + ' : ' + me.offsetCoords[1] + ' : ' + me.windowOffsetCoords[1]);
        }
    }
    /**
     * @return {?}
     */
    throttledUpdate() {
        let /** @type {?} */ me = this;
        if (me.throttle === undefined) {
            me.updateOffset();
            me.throttle = window.setTimeout(() => {
                me.updatePos();
                me.throttle = undefined;
            }, this.entry.delay);
        }
    }
    /**
     * @return {?}
     */
    updatePos() {
        let /** @type {?} */ me = this;
        let /** @type {?} */ coords = [
            Math.round(me.anchorCoords[0] + me.offsetCoords[0] + me.windowOffsetCoords[0]),
            Math.round(me.anchorCoords[1] + me.offsetCoords[1] + me.windowOffsetCoords[1])
        ];
        let /** @type {?} */ parent = me.ele.parentElement;
        if (parent) {
            me.ele.style.maxWidth = (parent.clientWidth - windowOffset * 2) + 'px';
            me.ele.style.maxHeight = (parent.clientHeight - windowOffset * 2) + 'px';
        }
        me.isDrawing = false;
        // dont move it if its already there
        if (coords[0] !== me.lastCoords[0] || coords[1] !== me.lastCoords[1]) {
            me.ele.style.left = (coords[0]) + 'px';
            me.ele.style.top = (coords[1]) + 'px';
            me.lastCoords = coords;
            me.drawTilNoChange();
            // ele width can change after rendering so check it again
        }
        else {
            this.ele.style.visibility = 'visible';
            this.checkInterval = window.setTimeout(() => { me.drawTilNoChange.apply(me); }, 1000 / 30);
        }
    }
    /**
     * @param {?} cb
     * @return {?}
     */
    renderThen(cb) {
        let /** @type {?} */ me = this;
        setTimeout(() => {
            // reset window offset
            cb.apply(me);
        }, 100);
    }
    /**
     * @return {?}
     */
    drawTilNoChange() {
        let /** @type {?} */ me = this;
        if (me.isDrawing) {
            return;
        }
        me.isDrawing = true;
        me.renderThen(() => {
            me.updateOffset();
            me.ele.style.width = me.ele.clientWidth + 'px';
            me.renderThen(() => {
                me.updateWindowOffset();
                me.updatePos();
            });
        });
    }
}
GistPopoverDisplayerComponent.decorators = [
    { type: Component, args: [{
                selector: 'gist-popover-displayer',
                template: `<ng-container #target>
</ng-container>
`,
                styles: [`:host {
  display: block;
  position: absolute;
  background: white;
  border-radius: 5px;
  -webkit-box-shadow: 0 3px 1px -2px rgba(0, 0, 0, .2), 0 2px 2px 0 rgba(0, 0, 0, .14), 0 1px 5px 0 rgba(0, 0, 0, .12);
          box-shadow: 0 3px 1px -2px rgba(0, 0, 0, .2), 0 2px 2px 0 rgba(0, 0, 0, .14), 0 1px 5px 0 rgba(0, 0, 0, .12);
  -webkit-transition: left 0.1s ease;
  transition: left 0.1s ease;
  -webkit-transition: top 0.1s ease;
  transition: top 0.1s ease;
}
`]
            },] },
];
/** @nocollapse */
GistPopoverDisplayerComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: ChangeDetectorRef, },
];
GistPopoverDisplayerComponent.propDecorators = {
    "entry": [{ type: Input },],
    "target": [{ type: ViewChild, args: ['target', { read: ViewContainerRef },] },],
    "onMouseEnter": [{ type: HostListener, args: ['mouseenter',] },],
    "onMouseLeave": [{ type: HostListener, args: ['mouseleave',] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class GistPopoverModule {
}
GistPopoverModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                declarations: [
                    GistPopoverDisplayerComponent,
                    GistPopoverTargetComponent,
                    GistPopoverDirective
                ],
                providers: [
                    GistPopoverService
                ],
                exports: [
                    GistPopoverDirective,
                    GistPopoverTargetComponent
                ]
            },] },
];
/** @nocollapse */
GistPopoverModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Generated bundle index. Do not edit.
 */

export { GistPopoverModule, GistPopoverDisplayerComponent as ɵa, GistPopoverTargetComponent as ɵb, GistPopoverDirective as ɵd, GistPopoverService as ɵc };
//# sourceMappingURL=c5-angular-popovers.js.map
