import { ChangeDetectorRef, Component, Directive, ElementRef, HostListener, Injectable, Input, NgModule, TemplateRef, ViewChild, ViewContainerRef } from '@angular/core';
import { BehaviorSubject as BehaviorSubject$1 } from 'rxjs/BehaviorSubject';
import { Observable } from 'rxjs';
import { CommonModule } from '@angular/common';
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var nextId = 0;
var PopoverEntry = (function () {
    /**
     * @param {?} tempRef
     */
    function PopoverEntry(tempRef) {
        this.anchorSide = new BehaviorSubject$1('bottom');
        this.xPos = new BehaviorSubject$1(0);
        this.yPos = new BehaviorSubject$1(0);
        this.isInteractive = false;
        this.delay = 0;
        this.isShown = false;
        this.isActive = false;
        this.template = tempRef;
        nextId++;
        this._id = nextId;
    }
    Object.defineProperty(PopoverEntry.prototype, "id", {
        /**
         * @return {?}
         */
        get: function () { return this._id; },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    PopoverEntry.prototype.destroy = function () {
        this.anchorSide.complete();
        this.xPos.complete();
        this.yPos.complete();
    };
    return PopoverEntry;
}());
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var GistPopoverService = (function () {
    function GistPopoverService() {
        this._popovers = [];
        this.idIndexMap = {};
    }
    /**
     * @param {?} templateRef
     * @return {?}
     */
    GistPopoverService.prototype.registerPopover = function (templateRef) {
        var /** @type {?} */ entry = new PopoverEntry(templateRef);
        this.idIndexMap[entry.id] = this._popovers.length;
        this._popovers.push(entry);
        return entry;
    };
    /**
     * @param {?} entry
     * @return {?}
     */
    GistPopoverService.prototype.unRegisterPopover = function (entry) {
        var /** @type {?} */ targetIndex = this.idIndexMap[entry.id];
        this._popovers.splice(targetIndex, 1);
        entry.destroy();
    };
    return GistPopoverService;
}());
GistPopoverService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
GistPopoverService.ctorParameters = function () { return []; };
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var offset = 15;
var eleMoveTrackTime = 1000 / 30;
var ɵ0 = function (val) {
    var /** @type {?} */ opts = ['ele', 'mouse']; // or x,y where x and y are numbers
    var /** @type {?} */ isValid = false;
    if (opts.indexOf(val) !== -1) {
        isValid = true;
    }
    else if (val.indexOf(',')) {
        var /** @type {?} */ coords = val.split(',');
        if (coords.length === 2) {
            var /** @type {?} */ isXnan = isNaN(parseFloat(coords[0]));
            var /** @type {?} */ isYnan = isNaN(parseFloat(coords[1]));
            isValid = !isXnan && !isYnan;
        }
    }
    if (!isValid) {
        console.error('gistPopover invalid paramater value. Target must be one of ' + opts.join(', ') + ', XX,YY ( as page coordinates). Value given: ' + val);
        return;
    }
    return val;
};
var ɵ1 = function (val) {
    var /** @type {?} */ opts = ['top', 'bottom', 'left', 'right'];
    if (opts.indexOf(val) === -1) {
        console.error('gistPopover invalid paramater value. Position must be one of ' + opts.join(', ') + '. Value given: ' + val);
        return;
    }
    return val;
};
var ɵ2 = function (val) {
    if (val === 'true') {
        return true;
    }
    else if (val === 'false') {
        return false;
    }
    else {
        console.error('gistPopover invalid paramater value. Position must be the string "true" or "false". Value given: ' + val);
    }
};
var ɵ3 = function (val) {
    if (val === 'true') {
        return true;
    }
    else if (val === 'false') {
        return false;
    }
    else if (val === 'undefined') {
        return undefined;
    }
    else {
        console.error('gistPopover invalid paramater value. Position must be the string "true", "false", or "undefined". Value given: ' + val);
        return;
    }
};
var ɵ4 = function (val) {
    var /** @type {?} */ tmp = +val;
    if (tmp.toString() !== val.toString() || tmp < 0) {
        console.error('gistPopover invalid paramater value. Delay must be a number >= 0');
        return;
    }
};
var validInputs = [
    {
        key: 'target',
        default: 'ele',
        parse: ɵ0,
    },
    {
        key: 'position',
        default: 'bottom',
        parse: ɵ1
    },
    {
        key: 'isInteractive',
        default: false,
        parse: ɵ2
    },
    {
        key: 'isVisible',
        default: undefined,
        parse: ɵ3
    },
    {
        key: 'delay',
        default: 100,
        parse: ɵ4
    }
];
var GistPopoverDirective = (function () {
    /**
     * @param {?} templateRef
     * @param {?} viewContainer
     * @param {?} popService
     * @param {?} eleRef
     * @param {?} cdr
     */
    function GistPopoverDirective(templateRef, viewContainer, popService, eleRef, cdr) {
        this.templateRef = templateRef;
        this.viewContainer = viewContainer;
        this.popService = popService;
        this.eleRef = eleRef;
        this.cdr = cdr;
        this.windowSubs = [];
        this.mouseSubs = [];
        this.isReady = false;
        this.hideTimer = undefined;
        this.showTimer = undefined;
        this.checkTimer = undefined;
        var /** @type {?} */ me = this;
        me.entry = me.popService.registerPopover(me.templateRef);
    }
    /**
     * @return {?}
     */
    GistPopoverDirective.prototype.ngAfterViewInit = function () {
        var /** @type {?} */ me = this;
        me.ele = me.eleRef.nativeElement;
        // take the content out of the view
        me.viewContainer.clear();
        me.windowSubs.push(Observable.fromEvent(window.document, 'resize').subscribe(function (e) { return me.updatePopover.apply(me); }));
        me.isReady = true;
    };
    Object.defineProperty(GistPopoverDirective.prototype, "gistPopover", {
        /**
         * @param {?} inputString
         * @return {?}
         */
        set: function (inputString) {
            var /** @type {?} */ me = this;
            var /** @type {?} */ inputs = inputString ? inputString.split(';') : [];
            validInputs.forEach(function (vi) { me[vi.key] = undefined; });
            inputs.forEach(function (input) {
                // reset inputs
                var /** @type {?} */ pv = input.split(':');
                if (pv.length !== 2) {
                    return;
                }
                var /** @type {?} */ prop = pv[0].trim();
                var /** @type {?} */ val = pv[1].trim();
                var /** @type {?} */ myProp = validInputs.filter(function (vi) { return vi.key === prop; })[0];
                if (!myProp) {
                    throw 'gistPopover invalid property: ' + prop + '. ' +
                        'Valid properties are ' + validInputs.map(function (vi) { return vi.key; }).join(', ');
                }
                try {
                    me[prop] = myProp.parse(val);
                }
                catch (e) {
                    console.error(e);
                    me[prop] = undefined;
                }
            });
            validInputs.forEach(function (vi) {
                if (me[vi.key] === undefined) {
                    me[vi.key] = vi.default;
                }
            });
            me.waitTillReady(me.processInput);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    GistPopoverDirective.prototype.processInput = function () {
        var /** @type {?} */ me = this;
        // track mouse if isVisisble is not defiend
        if (me.target === 'mouse' || me.isVisible === undefined) {
            // start listening if were not
            if (me.mouseSubs.length === 0) {
                me.trackMouse();
            }
        }
        else {
            if (me.mouseSubs.length !== 0) {
                me.unTrackMouse();
            }
        }
        if (me.isVisible !== undefined) {
            if (me.isVisible) {
                me.showPopover();
            }
            else {
                me.hidePopover();
            }
        }
        if (me.isVisible && me.target.indexOf(',') !== -1) {
            me.updatePopover();
        }
    };
    /**
     * @return {?}
     */
    GistPopoverDirective.prototype.trackMouse = function () {
        var /** @type {?} */ me = this;
        var /** @type {?} */ pe = me.ele.parentElement;
        if (pe) {
            me.mouseSubs.push(Observable.fromEvent(pe, 'mousemove').subscribe(function (e) {
                if (me.isVisible || me.isVisible === undefined) {
                    me.showPopover.apply(me, [e]);
                }
            }));
            me.mouseSubs.push(Observable.fromEvent(pe, 'mouseleave').subscribe(function (e) {
                if (!me.isVisible) {
                    me.hidePopover.apply(me);
                }
            }));
        }
        else {
            console.warn('no element to listen to.');
        }
    };
    /**
     * @return {?}
     */
    GistPopoverDirective.prototype.unTrackMouse = function () {
        this.mouseSubs.forEach(function (s) { return s.unsubscribe(); });
    };
    /**
     * @param {?=} e
     * @return {?}
     */
    GistPopoverDirective.prototype.updatePopover = function (e) {
        if (this.entry && this.entry.isShown) {
            this.showPopover(e);
        }
    };
    /**
     * @param {?=} e
     * @return {?}
     */
    GistPopoverDirective.prototype.showPopover = function (e) {
        var _this = this;
        var /** @type {?} */ me = this;
        if (me.hideTimer) {
            window.clearTimeout(me.hideTimer);
            me.hideTimer = undefined;
        }
        /**
         * @param {?} coords
         * @return {?}
         */
        function display(coords) {
            me.entry.xPos.next(coords[0]);
            me.entry.yPos.next(coords[1]);
        }
        me.entry.delay = me.delay;
        me.entry.isInteractive = me.isInteractive;
        me.entry.anchorSide.next(me.position);
        var /** @type {?} */ coords = undefined;
        if (me.target === 'mouse') {
            if (e) {
                coords = me.getMouseAnchor(e);
            }
        }
        else if (me.target === 'ele') {
            coords = me.getEleAnchor(/** @type {?} */ (me.ele.parentElement));
        }
        else {
            coords = me.getTargetAnchor();
        }
        if (coords) {
            display(coords);
            if (!this.showTimer) {
                this.showTimer = window.setTimeout(function () {
                    me.entry.isShown = true;
                    me.showTimer = undefined;
                    _this.checkTimer = window.setTimeout(function () {
                        me.showPopover.apply(me, [e]);
                    }, eleMoveTrackTime);
                    _this.cdr.detectChanges();
                }, me.entry.delay);
            }
        }
        // debugger;
    };
    /**
     * @param {?} event
     * @return {?}
     */
    GistPopoverDirective.prototype.getMouseAnchor = function (event) {
        var /** @type {?} */ xPos = event.pageX;
        var /** @type {?} */ yPos = event.pageY;
        var /** @type {?} */ xOffset = offset;
        var /** @type {?} */ yOffset = offset;
        if (this.position === 'left') {
            xOffset = offset * -1;
        }
        if (this.position === 'top') {
            yOffset = offset * -1;
        }
        return [xPos + xOffset, yPos + yOffset];
    };
    /**
     * @param {?} ele
     * @return {?}
     */
    GistPopoverDirective.prototype.getEleAnchor = function (ele) {
        var /** @type {?} */ bounds = ele.getBoundingClientRect();
        var /** @type {?} */ xPos = bounds.left + bounds.width / 2;
        var /** @type {?} */ yPos = bounds.top + bounds.height / 2;
        if (this.position === 'left') {
            xPos = bounds.left - offset;
        }
        else if (this.position === 'right') {
            xPos = bounds.right + offset;
        }
        if (this.position === 'top') {
            yPos = bounds.top - offset;
        }
        else if (this.position === 'bottom') {
            yPos = bounds.bottom + offset;
        }
        return [xPos, yPos];
    };
    /**
     * @return {?}
     */
    GistPopoverDirective.prototype.getTargetAnchor = function () {
        return this.target.split(',').map(function (x) { return +x; });
    };
    /**
     * @return {?}
     */
    GistPopoverDirective.prototype.hidePopover = function () {
        var _this = this;
        var /** @type {?} */ me = this;
        if (this.showTimer !== undefined) {
            clearTimeout(this.showTimer);
            this.showTimer = undefined;
        }
        if (this.checkTimer !== undefined) {
            clearTimeout(this.checkTimer);
            this.checkTimer = undefined;
        }
        if (me.entry.isShown && !this.hideTimer) {
            this.hideTimer = window.setTimeout(function () {
                _this.entry.isShown = false;
                _this.hideTimer = undefined;
            }, this.entry.delay);
        }
    };
    /**
     * @return {?}
     */
    GistPopoverDirective.prototype.ngOnDestroy = function () {
        this.popService.unRegisterPopover(this.entry);
        this.mouseSubs.forEach(function (s) { return s.unsubscribe(); });
        this.windowSubs.forEach(function (s) { return s.unsubscribe(); });
    };
    /**
     * @param {?} cb
     * @return {?}
     */
    GistPopoverDirective.prototype.waitTillReady = function (cb) {
        var /** @type {?} */ me = this;
        if (me.isReady) {
            cb.apply(me);
        }
        else {
            window.setTimeout(function () {
                me.waitTillReady.apply(me, [cb]);
            }, 1);
        }
    };
    return GistPopoverDirective;
}());
GistPopoverDirective.decorators = [
    { type: Directive, args: [{ selector: '[gistPopover]' },] },
];
/** @nocollapse */
GistPopoverDirective.ctorParameters = function () { return [
    { type: TemplateRef, },
    { type: ViewContainerRef, },
    { type: GistPopoverService, },
    { type: ElementRef, },
    { type: ChangeDetectorRef, },
]; };
GistPopoverDirective.propDecorators = {
    "gistPopover": [{ type: Input },],
};
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var GistPopoverTargetComponent = (function () {
    /**
     * @param {?} popService
     */
    function GistPopoverTargetComponent(popService) {
        this.popService = popService;
    }
    return GistPopoverTargetComponent;
}());
GistPopoverTargetComponent.decorators = [
    { type: Component, args: [{
                selector: 'gist-popover-target',
                template: "<ng-container *ngFor=\"let entry of popService._popovers\">\n  <ng-container *ngIf=\"entry.isShown || entry.isActive\">\n    <gist-popover-displayer [entry]=\"entry\" [ngClass]=\"{'interactive': entry.isInteractive}\"> </gist-popover-displayer>\n  </ng-container>\n</ng-container>\n",
                styles: [":host {\n  display: block;\n  position: fixed;\n  top: 0;\n  bottom: 0;\n  right: 0;\n  left: 0;\n  pointer-events: none;\n}\n\n.interactive {\n  pointer-events: all;\n}\n"]
            },] },
];
/** @nocollapse */
GistPopoverTargetComponent.ctorParameters = function () { return [
    { type: GistPopoverService, },
]; };
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var windowOffset = 5;
var GistPopoverDisplayerComponent = (function () {
    /**
     * @param {?} eleRef
     * @param {?} cdr
     */
    function GistPopoverDisplayerComponent(eleRef, cdr) {
        this.eleRef = eleRef;
        this.cdr = cdr;
        this.viewTimer = undefined;
        this.anchorCoords = [0, 0];
        this.offsetCoords = [0, 0];
        this.windowOffsetCoords = [0, 0];
        this.lastCoords = [0, 0];
        this.throttle = undefined;
        this.isDrawing = false;
        this.ele = this.eleRef.nativeElement;
        this.ele.style.visibility = 'hidden';
    }
    /**
     * @return {?}
     */
    GistPopoverDisplayerComponent.prototype.onMouseEnter = function () {
        this.entry.isActive = true;
        if (this.viewTimer) {
            window.clearTimeout(this.viewTimer);
            this.viewTimer = undefined;
        }
    };
    /**
     * @return {?}
     */
    GistPopoverDisplayerComponent.prototype.onMouseLeave = function () {
        var _this = this;
        if (this.viewTimer === undefined) {
            this.viewTimer = window.setTimeout(function () {
                _this.entry.isActive = false;
                _this.viewTimer = undefined;
            }, this.entry.delay);
        }
    };
    /**
     * @return {?}
     */
    GistPopoverDisplayerComponent.prototype.ngAfterViewInit = function () {
        var /** @type {?} */ me = this;
        me.target.createEmbeddedView(me.entry.template);
        me.entry.anchorSide.subscribe(function (anc) {
            if (me.anchor !== anc) {
                me.anchor = anc;
                me.throttledUpdate();
            }
        });
        me.entry.xPos.subscribe(function (x) {
            if (me.anchorCoords[0] !== x) {
                me.anchorCoords[0] = x;
                me.throttledUpdate();
            }
        });
        me.entry.yPos.subscribe(function (y) {
            if (me.anchorCoords[1] !== y) {
                me.anchorCoords[1] = y;
                me.throttledUpdate();
            }
        });
        this.cdr.detectChanges();
    };
    /**
     * @return {?}
     */
    GistPopoverDisplayerComponent.prototype.ngOnDestroy = function () {
        window.clearTimeout(this.checkInterval);
    };
    /**
     * @return {?}
     */
    GistPopoverDisplayerComponent.prototype.updateOffset = function () {
        var /** @type {?} */ me = this;
        var /** @type {?} */ parent = me.ele.parentElement;
        if (!parent) {
            return;
        }
        if (me.anchor === 'left') {
            me.offsetCoords[0] = me.ele.clientWidth * -1;
            me.offsetCoords[1] = me.ele.clientHeight * -0.5;
        }
        else if (me.anchor === 'right') {
            me.offsetCoords[0] = 0;
            me.offsetCoords[1] = me.ele.clientHeight * -0.5;
        }
        else if (me.anchor === 'top') {
            me.offsetCoords[0] = me.ele.clientWidth * -0.5;
            me.offsetCoords[1] = me.ele.clientHeight * -1;
        }
        else if (me.anchor === 'bottom') {
            me.offsetCoords[0] = me.ele.clientWidth * -0.5;
            me.offsetCoords[1] = 0;
        }
        me.offsetCoords[0] = +me.offsetCoords[0].toFixed(2);
        me.offsetCoords[1] = +me.offsetCoords[1].toFixed(2);
    };
    /**
     * @return {?}
     */
    GistPopoverDisplayerComponent.prototype.updateWindowOffset = function () {
        var /** @type {?} */ me = this;
        var /** @type {?} */ parent = me.ele.parentElement;
        if (!parent) {
            return;
        }
        var /** @type {?} */ myBounds = me.ele.getBoundingClientRect();
        var /** @type {?} */ parentBounds = parent.getBoundingClientRect();
        var /** @type {?} */ height = myBounds.height;
        var /** @type {?} */ width = myBounds.width;
        var /** @type {?} */ x1 = me.anchorCoords[0] + me.offsetCoords[0];
        var /** @type {?} */ x2 = x1 + width;
        var /** @type {?} */ y1 = me.anchorCoords[1] + me.offsetCoords[1];
        var /** @type {?} */ y2 = y1 + height;
        if (x1 < windowOffset) {
            me.windowOffsetCoords[0] = windowOffset - x1;
        }
        if (x2 > parentBounds.right - windowOffset) {
            me.windowOffsetCoords[0] = parentBounds.right - windowOffset - x2;
        }
        if (y1 < windowOffset) {
            me.windowOffsetCoords[1] = windowOffset - y1;
        }
        if (y2 > parentBounds.bottom - windowOffset) {
            me.windowOffsetCoords[1] = parentBounds.bottom - windowOffset - y2;
        }
        me.windowOffsetCoords[0] = +me.windowOffsetCoords[0].toFixed(2);
        me.windowOffsetCoords[1] = +me.windowOffsetCoords[1].toFixed(2);
        if (me.anchorCoords[1] + me.offsetCoords[1] + me.windowOffsetCoords[1] < windowOffset) {
            console.error(me.anchorCoords[1] + ' : ' + me.offsetCoords[1] + ' : ' + me.windowOffsetCoords[1]);
        }
    };
    /**
     * @return {?}
     */
    GistPopoverDisplayerComponent.prototype.throttledUpdate = function () {
        var /** @type {?} */ me = this;
        if (me.throttle === undefined) {
            me.updateOffset();
            me.throttle = window.setTimeout(function () {
                me.updatePos();
                me.throttle = undefined;
            }, this.entry.delay);
        }
    };
    /**
     * @return {?}
     */
    GistPopoverDisplayerComponent.prototype.updatePos = function () {
        var /** @type {?} */ me = this;
        var /** @type {?} */ coords = [
            Math.round(me.anchorCoords[0] + me.offsetCoords[0] + me.windowOffsetCoords[0]),
            Math.round(me.anchorCoords[1] + me.offsetCoords[1] + me.windowOffsetCoords[1])
        ];
        var /** @type {?} */ parent = me.ele.parentElement;
        if (parent) {
            me.ele.style.maxWidth = (parent.clientWidth - windowOffset * 2) + 'px';
            me.ele.style.maxHeight = (parent.clientHeight - windowOffset * 2) + 'px';
        }
        me.isDrawing = false;
        // dont move it if its already there
        if (coords[0] !== me.lastCoords[0] || coords[1] !== me.lastCoords[1]) {
            me.ele.style.left = (coords[0]) + 'px';
            me.ele.style.top = (coords[1]) + 'px';
            me.lastCoords = coords;
            me.drawTilNoChange();
            // ele width can change after rendering so check it again
        }
        else {
            this.ele.style.visibility = 'visible';
            this.checkInterval = window.setTimeout(function () { me.drawTilNoChange.apply(me); }, 1000 / 30);
        }
    };
    /**
     * @param {?} cb
     * @return {?}
     */
    GistPopoverDisplayerComponent.prototype.renderThen = function (cb) {
        var /** @type {?} */ me = this;
        setTimeout(function () {
            // reset window offset
            cb.apply(me);
        }, 100);
    };
    /**
     * @return {?}
     */
    GistPopoverDisplayerComponent.prototype.drawTilNoChange = function () {
        var /** @type {?} */ me = this;
        if (me.isDrawing) {
            return;
        }
        me.isDrawing = true;
        me.renderThen(function () {
            me.updateOffset();
            me.ele.style.width = me.ele.clientWidth + 'px';
            me.renderThen(function () {
                me.updateWindowOffset();
                me.updatePos();
            });
        });
    };
    return GistPopoverDisplayerComponent;
}());
GistPopoverDisplayerComponent.decorators = [
    { type: Component, args: [{
                selector: 'gist-popover-displayer',
                template: "<ng-container #target>\n</ng-container>\n",
                styles: [":host {\n  display: block;\n  position: absolute;\n  background: white;\n  border-radius: 5px;\n  -webkit-box-shadow: 0 3px 1px -2px rgba(0, 0, 0, .2), 0 2px 2px 0 rgba(0, 0, 0, .14), 0 1px 5px 0 rgba(0, 0, 0, .12);\n          box-shadow: 0 3px 1px -2px rgba(0, 0, 0, .2), 0 2px 2px 0 rgba(0, 0, 0, .14), 0 1px 5px 0 rgba(0, 0, 0, .12);\n  -webkit-transition: left 0.1s ease;\n  transition: left 0.1s ease;\n  -webkit-transition: top 0.1s ease;\n  transition: top 0.1s ease;\n}\n"]
            },] },
];
/** @nocollapse */
GistPopoverDisplayerComponent.ctorParameters = function () { return [
    { type: ElementRef, },
    { type: ChangeDetectorRef, },
]; };
GistPopoverDisplayerComponent.propDecorators = {
    "entry": [{ type: Input },],
    "target": [{ type: ViewChild, args: ['target', { read: ViewContainerRef },] },],
    "onMouseEnter": [{ type: HostListener, args: ['mouseenter',] },],
    "onMouseLeave": [{ type: HostListener, args: ['mouseleave',] },],
};
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var GistPopoverModule = (function () {
    function GistPopoverModule() {
    }
    return GistPopoverModule;
}());
GistPopoverModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                declarations: [
                    GistPopoverDisplayerComponent,
                    GistPopoverTargetComponent,
                    GistPopoverDirective
                ],
                providers: [
                    GistPopoverService
                ],
                exports: [
                    GistPopoverDirective,
                    GistPopoverTargetComponent
                ]
            },] },
];
/** @nocollapse */
GistPopoverModule.ctorParameters = function () { return []; };
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Generated bundle index. Do not edit.
 */
export { GistPopoverModule, GistPopoverDisplayerComponent as ɵa, GistPopoverTargetComponent as ɵb, GistPopoverDirective as ɵd, GistPopoverService as ɵc };
//# sourceMappingURL=c5-angular-popovers.js.map
